/* Display a dialog for setting the wrap margin.
	93/12/24 aih - split into several functions so can use them in prefs dialog
	93/12/23 aih - uses preferences for setting units
	93/11/29 aih - split out of TextDocumentEventLib.c */

#include <stdio.h>
#include <string.h>
#include "ApplicationPreferencesLib.h"
#include "DialogModalLib.h"
#include "DrawLib.h"
#include "KeyLib.h"
#include "RadioLib.h"
#include "ResourceConstantsLib.h"
#include "TextWrapDialogLib.h"

#define PRECISION (2) /* precision of margin floating point value */

/* items in the wrap dialog */
enum {
	iDontWrapButton = 3,
	iWrapToMarginButton,
	iMarginText,
	iUnitsText
};

/* get settings from wrap dialog */
void TxWrapDialogValues(DialogPtr dlg, RadioHandle radio,
	short iwrap, short idontwrap, short imargin,
	Boolean *wrap, float *margin)
{
	*wrap = (RadioSelectedID(radio) == iwrap);
	*margin = DlgFloat(dlg, imargin) * AppPrefs()->units.conversion;
	if (*margin < TX_MIN_MARGIN)
		*margin = TX_MIN_MARGIN;
	else if (*margin > TX_MAX_MARGIN)
		*margin = TX_MAX_MARGIN;
}

/* set values of wrap dialog items */
void TxWrapDialogValuesSet(DialogPtr dlg, RadioHandle radio,
	short iwrap, short idontwrap, short imargin, short iunits,
	Boolean wrap, float margin)
{
	RadioClickID(radio, wrap ? iwrap : idontwrap);
	DlgTextSet(dlg, iunits, AppPrefs()->units.plural);
	DlgFloatSet(dlg, imargin, margin / AppPrefs()->units.conversion, PRECISION);
}

/* setup wrap dialog */
RadioHandle TxWrapDialogSetup(DialogPtr dlg,
	short iwrap, short idontwrap, short imargin, short iunits,
	short iframe, short ititle, Boolean wrap, float margin)
{
	volatile RadioHandle radio = NULL;
	Rect frame, title;
	
	TRY {
		if (iframe) DlgBox(dlg, iframe, &frame);
		if (ititle) DlgBox(dlg, ititle, &title);
		radio = RadioBegin(dlg, iframe ? &frame : NULL, ititle ? &title : NULL);
		RadioAdd(radio, DlgCtl(dlg, iwrap), iwrap);
		RadioAdd(radio, DlgCtl(dlg, idontwrap), idontwrap);
		TxHookKeySet(WinRegisteredID(dlg, imargin), TxHookKeyUnsignedFloat);
	} CATCH {
		RadioEnd(radio);
	} ENDTRY;
	return(radio);
}

/* display wrap margin dialog, return true if settings changed */
Boolean TxWrapDialog(TextHandle text)
{
	volatile DialogPtr dlg = NULL; /* wrap dialog */
	volatile RadioHandle	radio;	/* radio buttons */
	Boolean wrap;						/* true if word wrap is enabled */
	float margin;						/* current margin */
	short item;							/* item hit in dialog */
	Boolean changed = false;		/* true if changed settings */
	
	require(TxValid(text));
	TRY {

		/* create and setup dialog */
		dlg = DlgBegin(RLD_TXWRAP);
		wrap = TxWrap(text);
		margin = (wrap ? TxMargin(text) : AppPrefs()->text.margin);
		radio = TxWrapDialogSetup(dlg, iWrapToMarginButton, iDontWrapButton,
			iMarginText, iUnitsText, 0, 0, wrap, margin);
		TxWrapDialogValuesSet(dlg, radio, iWrapToMarginButton, iDontWrapButton,
			iMarginText, iUnitsText, wrap, margin);
		DlgTextSelectAll(dlg, iMarginText);
		WinShow(dlg);

		/* run dialog */
		do {
			item = DlgModalRun(dlg);
			if (item == iMarginText)
				RadioClickID(radio, iWrapToMarginButton);
		} while (item != ok && item != cancel);
				
		if (item == ok) {
			/* apply settings */
			DrawCursor(watchCursor);
			TxWrapDialogValues(dlg, radio, iWrapToMarginButton,
				iDontWrapButton, iMarginText, &wrap, &margin);
			changed = (TxWrap(text) != wrap || (wrap && TxMargin(text) != margin));
			TxWrapSet(text, wrap);
			TxMarginSet(text, margin);
			if (changed) {
				TxInval(text);
				TxCalculate(text);
			}
		}
	} CLEANUP {
		RadioEnd(radio);
		DlgEnd(dlg);
	} ENDTRY;
	return(changed);
}
